@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2008 Internet2
+ *  Copyright 2001-2009 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,17 +26,29 @@
 # include "config.h"
 #endif
 
+#ifdef WIN32
+# define _CRT_NONSTDC_NO_DEPRECATE 1
+# define _CRT_SECURE_NO_DEPRECATE 1
+# define MCEXT_EXPORTS __declspec(dllexport)
+#else
+# define MCEXT_EXPORTS
+#endif
+
+#include <xmltooling/base.h>
+
+#include <list>
+#include <iostream> 
+#include <libmemcached/memcached.h>
 #include <xercesc/util/XMLUniDefs.hpp>
 
 #include <xmltooling/logging.h>
-
+#include <xmltooling/unicode.h>
 #include <xmltooling/XMLToolingConfig.h>
 #include <xmltooling/util/NDC.h>
 #include <xmltooling/util/StorageService.h>
+#include <xmltooling/util/Threads.h>
 #include <xmltooling/util/XMLHelper.h>
 
-#include <libmemcached/memcached.h>
-
 using namespace xmltooling::logging;
 using namespace xmltooling;
 using namespace xercesc;
@@ -46,6 +58,11 @@ namespace xmltooling {
   static const XMLCh Hosts[] = UNICODE_LITERAL_5(H,o,s,t,s);
   static const XMLCh prefix[] = UNICODE_LITERAL_6(p,r,e,f,i,x);
   static const XMLCh buildMap[] = UNICODE_LITERAL_8(b,u,i,l,d,M,a,p);
+  static const XMLCh sendTimeout[] = UNICODE_LITERAL_11(s,e,n,d,T,i,m,e,o,u,t);
+  static const XMLCh recvTimeout[] = UNICODE_LITERAL_11(r,e,c,v,T,i,m,e,o,u,t);
+  static const XMLCh pollTimeout[] = UNICODE_LITERAL_11(p,o,l,l,T,i,m,e,o,u,t);
+  static const XMLCh failLimit[] = UNICODE_LITERAL_9(f,a,i,l,L,i,m,i,t);
+  static const XMLCh retryTimeout[] = UNICODE_LITERAL_12(r,e,t,r,y,T,i,m,e,o,u,t);
   
   class mc_record {
   public:
@@ -98,9 +115,8 @@ namespace xmltooling {
     const DOMElement* m_root; // can only use this during initialization
     Category& log;
     memcached_st *memc;
-    string m_memcacheHosts;
     string m_prefix;
-    
+    Mutex* m_lock;
   };
   
   class MemcacheStorageService : public StorageService, public MemcacheBase {
@@ -151,15 +167,19 @@ bool MemcacheBase::addLock(string what, bool use_prefix) {
   string set_val = "1";
   unsigned tries = 5;
   while (!addMemcache(lock_name.c_str(), set_val, 5, 0, use_prefix)) {
-    if (tries-- < 0) {
+    if (tries-- == 0) {
       log.debug("Unable to get lock %s... FAILED.", lock_name.c_str());
       return false;
     }
     log.debug("Unable to get lock %s... Retrying.", lock_name.c_str());
     
     // sleep 100ms
+#ifdef WIN32
+    Sleep(100);
+#else
     struct timeval tv = { 0, 100000 };
-    select(0, 0, 0, 0, &tv);    
+    select(0, 0, 0, 0, &tv);
+#endif
   }
   return true;
 }
@@ -264,7 +284,6 @@ bool MemcacheBase::deleteMemcache(const char *key,
                                   bool use_prefix) {
   memcached_return rv;
   string final_key;
-  memcached_st clone;
   bool success;
 
   if (use_prefix) {
@@ -273,23 +292,26 @@ bool MemcacheBase::deleteMemcache(const char *key,
     final_key = key;
   }
 
-  if (memcached_clone(&clone, memc) == NULL) {
-    throw IOException("MemcacheBase::deleteMemcache(): memcached_clone() failed");
-  }
+  m_lock->lock();
+  rv = memcached_delete(memc, (char *)final_key.c_str(), final_key.length(), timeout);
+  m_lock->unlock();
 
-  rv = memcached_delete(&clone, (char *)final_key.c_str(), final_key.length(), timeout);
   if (rv == MEMCACHED_SUCCESS) {
     success = true;
   } else if (rv == MEMCACHED_NOTFOUND) {
     // Key wasn't there... No biggie.
     success = false;
+  } else if (rv == MEMCACHED_ERRNO) {
+    // System error
+    string error = string("Memcache::deleteMemcache() SYSTEM ERROR: ") + string(strerror(memc->cached_errno));
+    log.error(error);
+    throw IOException(error);
   } else {
-    log.error(string("Memcache::deleteMemcache() Problems: ") + memcached_strerror(&clone, rv));
-    // shouldn't be here
-    success = false;
+    string error = string("Memcache::deleteMemcache() Problems: ") + memcached_strerror(memc, rv);
+    log.error(error);
+    throw IOException(error);
   }
 
-  memcached_free(&clone);
   return success;
 }
 
@@ -301,7 +323,6 @@ bool MemcacheBase::getMemcache(const char *key,
   size_t len;
   char *result;
   string final_key;
-  memcached_st clone;
   bool success;
   
   if (use_prefix) {
@@ -310,11 +331,10 @@ bool MemcacheBase::getMemcache(const char *key,
     final_key = key;
   }
 
-  if (memcached_clone(&clone, memc) == NULL) {
-    throw IOException("MemcacheBase::getMemcache(): memcached_clone() failed");
-  }
+  m_lock->lock();
+  result = memcached_get(memc, (char *)final_key.c_str(), final_key.length(), &len, flags, &rv);
+  m_lock->unlock();
 
-  result = memcached_get(&clone, (char *)final_key.c_str(), final_key.length(), &len, flags, &rv);
   if (rv == MEMCACHED_SUCCESS) {
     dest = result;
     free(result);
@@ -322,12 +342,17 @@ bool MemcacheBase::getMemcache(const char *key,
   } else if (rv == MEMCACHED_NOTFOUND) {
     log.debug("Key %s not found in memcache...", key);
     success = false;
+  } else if (rv == MEMCACHED_ERRNO) {
+    // System error
+    string error = string("Memcache::getMemcache() SYSTEM ERROR: ") + string(strerror(memc->cached_errno));
+    log.error(error);
+    throw IOException(error);
   } else {
-    log.error(string("Memcache::getMemcache() Problems: ") + memcached_strerror(&clone, rv));
-    success = false;
+    string error = string("Memcache::getMemcache() Problems: ") + memcached_strerror(memc, rv);
+    log.error(error);
+    throw IOException(error);
   }
 
-  memcached_free(&clone);
   return success;
 }
 
@@ -339,7 +364,6 @@ bool MemcacheBase::addMemcache(const char *key,
 
   memcached_return rv;
   string final_key;
-  memcached_st clone;
   bool success;
 
   if (use_prefix) {
@@ -348,23 +372,26 @@ bool MemcacheBase::addMemcache(const char *key,
     final_key = key;
   }
 
-  if (memcached_clone(&clone, memc) == NULL) {
-    throw IOException("MemcacheBase::addMemcache(): memcached_clone() failed");
-  }
+  m_lock->lock();
+  rv = memcached_add(memc, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
+  m_lock->unlock();
 
-  rv = memcached_add(&clone, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
   if (rv == MEMCACHED_SUCCESS) {
     success = true;
   } else if (rv == MEMCACHED_NOTSTORED) {
     // already there
     success = false;
+  } else if (rv == MEMCACHED_ERRNO) {
+    // System error
+    string error = string("Memcache::addMemcache() SYSTEM ERROR: ") + string(strerror(memc->cached_errno));
+    log.error(error);
+    throw IOException(error);
   } else {
-    // shouldn't be here
-    log.error(string("Memcache::addMemcache() Problems: ") + memcached_strerror(&clone, rv));
-    success = false;
+    string error = string("Memcache::addMemcache() Problems: ") + memcached_strerror(memc, rv);
+    log.error(error);
+    throw IOException(error);
   }
 
-  memcached_free(&clone);
   return success;
 }
 
@@ -376,7 +403,6 @@ bool MemcacheBase::setMemcache(const char *key,
 
   memcached_return rv;
   string final_key;
-  memcached_st clone;
   bool success;
 
   if (use_prefix) {
@@ -385,20 +411,23 @@ bool MemcacheBase::setMemcache(const char *key,
     final_key = key;
   }
 
-  if (memcached_clone(&clone, memc) == NULL) {
-    throw IOException("MemcacheBase::setMemcache(): memcached_clone() failed");
-  }
+  m_lock->lock();
+  rv = memcached_set(memc, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
+  m_lock->unlock();
 
-  rv = memcached_set(&clone, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
   if (rv == MEMCACHED_SUCCESS) {
     success = true;
+  } else if (rv == MEMCACHED_ERRNO) {
+    // System error
+    string error = string("Memcache::setMemcache() SYSTEM ERROR: ") + string(strerror(memc->cached_errno));
+    log.error(error);
+    throw IOException(error);
   } else {
-    // shouldn't be here
-    log.error(string("Memcache::setMemcache() Problems: ") + memcached_strerror(&clone, rv));
-    success = false;
+    string error = string("Memcache::setMemcache() Problems: ") + memcached_strerror(memc, rv);
+    log.error(error);
+    throw IOException(error);
   }
 
-  memcached_free(&clone);
   return success;
 }
 
@@ -410,7 +439,6 @@ bool MemcacheBase::replaceMemcache(const char *key,
   
   memcached_return rv;
   string final_key;
-  memcached_st clone;
   bool success;
 
   if (use_prefix) {
@@ -419,27 +447,30 @@ bool MemcacheBase::replaceMemcache(const char *key,
     final_key = key;
   }
 
-  if (memcached_clone(&clone, memc) == NULL) {
-    throw IOException("MemcacheBase::replaceMemcache(): memcached_clone() failed");
-  }
+  m_lock->lock();
+  rv = memcached_replace(memc, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
+  m_lock->unlock();
 
-  rv = memcached_replace(&clone, (char *)final_key.c_str(), final_key.length(), (char *)value.c_str(), value.length(), timeout, flags);
   if (rv == MEMCACHED_SUCCESS) {
     success = true;
   } else if (rv == MEMCACHED_NOTSTORED) {
     // not there
     success = false;
+  } else if (rv == MEMCACHED_ERRNO) {
+    // System error
+    string error = string("Memcache::replaceMemcache() SYSTEM ERROR: ") + string(strerror(memc->cached_errno));
+    log.error(error);
+    throw IOException(error);
   } else {
-    // shouldn't be here
-    log.error(string("Memcache::replaceMemcache() Problems: ") + memcached_strerror(&clone, rv));
-    success = false;
+    string error = string("Memcache::replaceMemcache() Problems: ") + memcached_strerror(memc, rv);
+    log.error(error);
+    throw IOException(error);
   }
 
-  memcached_free(&clone);
   return success;
 }
 
-MemcacheBase::MemcacheBase(const DOMElement* e) : m_root(e), log(Category::getInstance("XMLTooling.MemcacheBase")), m_memcacheHosts(""), m_prefix("") {
+MemcacheBase::MemcacheBase(const DOMElement* e) : m_root(e), log(Category::getInstance("XMLTooling.MemcacheBase")), m_prefix("") {
 
   auto_ptr_char p(e ? e->getAttributeNS(NULL,prefix) : NULL);
   if (p.get() && *p.get()) {
@@ -447,14 +478,8 @@ MemcacheBase::MemcacheBase(const DOMElement* e) : m_root(e), log(Category::getIn
     m_prefix = p.get();
   }
 
-  // Grab hosts from the configuration.
-  e = e ? XMLHelper::getFirstChildElement(e,Hosts) : NULL;
-  if (!e || !e->hasChildNodes()) {
-    throw XMLToolingException("Memcache StorageService requires Hosts element in configuration.");
-  }
-  auto_ptr_char h(e->getFirstChild()->getNodeValue());
-  log.debug("INIT: GOT Hosts: %s", h.get());
-  m_memcacheHosts = h.get();
+  m_lock = Mutex::create();
+  log.debug("Lock created");
 
   memc = memcached_create(NULL);
   if (memc == NULL) {
@@ -463,12 +488,59 @@ MemcacheBase::MemcacheBase(const DOMElement* e) : m_root(e), log(Category::getIn
 
   log.debug("Memcache created");
 
-  unsigned int set = MEMCACHED_HASH_CRC;
-  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_HASH, set);
+  unsigned int hash = MEMCACHED_HASH_CRC;
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_HASH, hash);
   log.debug("CRC hash set");
 
+  int32_t send_timeout = 1000000;
+  const XMLCh* tag = e ? e->getAttributeNS(NULL, sendTimeout) : NULL;
+  if (tag && *tag) {
+    send_timeout = XMLString::parseInt(tag);
+  }
+  log.debug("MEMCACHED_BEHAVIOR_SND_TIMEOUT will be set to %d", send_timeout);
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_SND_TIMEOUT, send_timeout);
+
+  int32_t recv_timeout = 1000000;
+  tag = e ? e->getAttributeNS(NULL, sendTimeout) : NULL;
+  if (tag && *tag) {
+    recv_timeout = XMLString::parseInt(tag);
+  }
+  log.debug("MEMCACHED_BEHAVIOR_RCV_TIMEOUT will be set to %d", recv_timeout);
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_RCV_TIMEOUT, recv_timeout);
+
+  int32_t poll_timeout = 1000;
+  tag = e ? e->getAttributeNS(NULL, pollTimeout) : NULL;
+  if (tag && *tag) {
+    poll_timeout = XMLString::parseInt(tag);
+  }
+  log.debug("MEMCACHED_BEHAVIOR_POLL_TIMEOUT will be set to %d", poll_timeout);
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_POLL_TIMEOUT, poll_timeout);
+
+  int32_t fail_limit = 5;
+  tag = e ? e->getAttributeNS(NULL, failLimit) : NULL;
+  if (tag && *tag) {
+    fail_limit = XMLString::parseInt(tag);
+  }
+  log.debug("MEMCACHED_BEHAVIOR_SERVER_FAILURE_LIMIT will be set to %d", fail_limit);
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_SERVER_FAILURE_LIMIT, fail_limit);
+
+  int32_t retry_timeout = 30;
+  tag = e ? e->getAttributeNS(NULL, retryTimeout) : NULL;
+  if (tag && *tag) {
+    retry_timeout = XMLString::parseInt(tag);
+  }
+  log.debug("MEMCACHED_BEHAVIOR_RETRY_TIMEOUT will be set to %d", retry_timeout);
+  memcached_behavior_set(memc, MEMCACHED_BEHAVIOR_RETRY_TIMEOUT, retry_timeout);
+
+  // Grab hosts from the configuration.
+  e = e ? XMLHelper::getFirstChildElement(e,Hosts) : NULL;
+  if (!e || !e->hasChildNodes()) {
+    throw XMLToolingException("Memcache StorageService requires Hosts element in configuration.");
+  }
+  auto_ptr_char h(e->getFirstChild()->getNodeValue());
+  log.debug("INIT: GOT Hosts: %s", h.get());
   memcached_server_st *servers;
-  servers = memcached_servers_parse((char *)m_memcacheHosts.c_str());
+  servers = memcached_servers_parse(const_cast<char*>(h.get()));
   log.debug("Got %u hosts.",  memcached_server_list_count(servers));
   if (memcached_server_push(memc, servers) != MEMCACHED_SUCCESS) {
     throw IOException("MemcacheBase::Memcache(): memcached_server_push() failed");    
@@ -480,6 +552,7 @@ MemcacheBase::MemcacheBase(const DOMElement* e) : m_root(e), log(Category::getIn
 
 MemcacheBase::~MemcacheBase() {
   memcached_free(memc);
+  delete m_lock;
   log.debug("Base object destroyed");
 }
 
@@ -564,6 +637,19 @@ int MemcacheStorageService::readString(const char* context, const char* key, str
   uint32_t rec_version;
   string value;
 
+  if (m_buildMap) {
+    log.debug("Checking context");
+
+    string map_name = context;
+    string ser_arr;
+    uint32_t flags;
+    bool ctx_found = getMemcache(map_name.c_str(), ser_arr, &flags);
+
+    if (!ctx_found) {
+      return 0;
+    }
+  }
+
   bool found = getMemcache(final_key.c_str(), value, &rec_version);
   if (!found) {
     return 0;
@@ -645,12 +731,6 @@ void MemcacheStorageService::updateContext(const char* context, time_t expiratio
   }
 
   string map_name = context;
-  
-  if (! addLock(map_name)) {
-    log.error("Unable to get lock for context %s!", context);
-    return;
-  }
-  
   string ser_arr;
   uint32_t flags;
   bool result = getMemcache(map_name.c_str(), ser_arr, &flags);
@@ -681,8 +761,6 @@ void MemcacheStorageService::updateContext(const char* context, time_t expiratio
     replaceMemcache(map_name.c_str(), ser_arr, expiration, flags);
   }
   
-  deleteLock(map_name);
-  
 }
 
 void MemcacheStorageService::deleteContext(const char* context) {
@@ -695,12 +773,6 @@ void MemcacheStorageService::deleteContext(const char* context) {
   }
 
   string map_name = context;
-  
-  if (! addLock(map_name)) {
-    log.error("Unable to get lock for context %s!", context);
-    return;
-  }
-  
   string ser_arr;
   uint32_t flags;
   bool result = getMemcache(map_name.c_str(), ser_arr, &flags);
@@ -723,16 +795,14 @@ void MemcacheStorageService::deleteContext(const char* context) {
     deleteMemcache(map_name.c_str(), 0);
   }
   
-  deleteLock(map_name);
-
 }
 
-extern "C" int xmltooling_extension_init(void*) {
+extern "C" int MCEXT_EXPORTS xmltooling_extension_init(void*) {
     // Register this SS type
     XMLToolingConfig::getConfig().StorageServiceManager.registerFactory("MEMCACHE", MemcacheStorageServiceFactory);
     return 0;
 }
 
-extern "C" void xmltooling_extension_term() {
+extern "C" void MCEXT_EXPORTS xmltooling_extension_term() {
     XMLToolingConfig::getConfig().StorageServiceManager.deregisterFactory("MEMCACHE");
 }
