@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,10 +24,14 @@
 #define __shibsp_config_h__
 
 #include <shibsp/base.h>
+
+#include <string>
 #ifndef SHIBSP_LITE
 # include <saml/binding/MessageDecoder.h>
+# include <saml/binding/MessageEncoder.h>
 #endif
 #include <xmltooling/PluginManager.h>
+#include <xmltooling/QName.h>
 #include <xercesc/dom/DOM.hpp>
 
 /**
@@ -65,13 +69,9 @@ namespace shibsp {
     {
         MAKE_NONCOPYABLE(SPConfig);
     public:
-        SPConfig() : attribute_value_delimeter(';'), m_serviceProvider(NULL),
-#ifndef SHIBSP_LITE
-            m_artifactResolver(NULL),
-#endif
-            m_features(0) {}
+        SPConfig();
 
-        virtual ~SPConfig() {}
+        virtual ~SPConfig();
 
         /**
          * Returns the global configuration object for the library.
@@ -104,9 +104,7 @@ namespace shibsp {
          *
          * @param enabled   bitmask of component constants
          */
-        void setFeatures(unsigned long enabled) {
-            m_features = enabled;
-        }
+        void setFeatures(unsigned long enabled);
 
         /**
          * Test whether a subsystem is enabled.
@@ -114,9 +112,7 @@ namespace shibsp {
          * @param feature   subsystem/component to test
          * @return true iff feature is enabled
          */
-        bool isEnabled(components_t feature) {
-            return (m_features & feature)>0;
-        }
+        bool isEnabled(components_t feature);
 
         /**
          * Initializes library
@@ -152,9 +148,7 @@ namespace shibsp {
          *
          * @return  global ServiceProvider or NULL
          */
-        ServiceProvider* getServiceProvider() const {
-            return m_serviceProvider;
-        }
+        ServiceProvider* getServiceProvider() const;
 
         /**
          * Instantiates and installs a ServiceProvider instance based on an XML configuration string
@@ -175,19 +169,14 @@ namespace shibsp {
          *
          * @param artifactResolver   new ArtifactResolver instance to store
          */
-        void setArtifactResolver(opensaml::MessageDecoder::ArtifactResolver* artifactResolver) {
-            delete m_artifactResolver;
-            m_artifactResolver = artifactResolver;
-        }
+        void setArtifactResolver(opensaml::MessageDecoder::ArtifactResolver* artifactResolver);
 
         /**
          * Returns the global ArtifactResolver instance.
          *
          * @return  global ArtifactResolver or NULL
          */
-        opensaml::MessageDecoder::ArtifactResolver* getArtifactResolver() const {
-            return m_artifactResolver;
-        }
+        const opensaml::MessageDecoder::ArtifactResolver* getArtifactResolver() const;
 #endif
 
         /** Separator for serialized values of multi-valued attributes. */
@@ -291,6 +280,7 @@ namespace shibsp {
 
     private:
         unsigned long m_features;
+        xercesc::DOMDocument* m_configDoc;
     };
 
 #if defined (_MSC_VER)
