@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 /**
  * ScopedAttributeDecoder.cpp
  *
- * Decodes SAML into ScopedAttributes
+ * Decodes SAML into ScopedAttributes.
  */
 
 #include "internal.h"
@@ -35,15 +35,15 @@ using namespace std;
 
 namespace shibsp {
     static const XMLCh Scope[] =            UNICODE_LITERAL_5(S,c,o,p,e);
-    static const XMLCh scopeDelimeter[] =   UNICODE_LITERAL_14(s,c,o,p,e,D,e,l,i,m,e,t,e,r);
+    static const XMLCh scopeDelimiter[] =   UNICODE_LITERAL_14(s,c,o,p,e,D,e,l,i,m,i,t,e,r);
 
     class SHIBSP_DLLLOCAL ScopedAttributeDecoder : virtual public AttributeDecoder
     {
     public:
-        ScopedAttributeDecoder(const DOMElement* e) : AttributeDecoder(e), m_delimeter('@') {
-            if (e && e->hasAttributeNS(NULL,scopeDelimeter)) {
-                auto_ptr_char d(e->getAttributeNS(NULL,scopeDelimeter));
-                m_delimeter = *(d.get());
+        ScopedAttributeDecoder(const DOMElement* e) : AttributeDecoder(e), m_delimiter('@') {
+            if (e && e->hasAttributeNS(NULL,scopeDelimiter)) {
+                auto_ptr_char d(e->getAttributeNS(NULL,scopeDelimiter));
+                m_delimiter = *(d.get());
             }
         }
         ~ScopedAttributeDecoder() {}
@@ -53,7 +53,7 @@ namespace shibsp {
             ) const;
 
     private:
-        char m_delimeter;
+        char m_delimiter;
     };
 
     AttributeDecoder* SHIBSP_DLLLOCAL ScopedAttributeDecoderFactory(const DOMElement* const & e)
@@ -69,9 +69,8 @@ shibsp::Attribute* ScopedAttributeDecoder::decode(
     char* val;
     char* scope;
     const XMLCh* xmlscope;
-    QName scopeqname(NULL,Scope);
-    auto_ptr<ScopedAttribute> scoped(new ScopedAttribute(ids, m_delimeter));
-    scoped->setCaseSensitive(m_caseSensitive);
+    xmltooling::QName scopeqname(NULL,Scope);
+    auto_ptr<ScopedAttribute> scoped(new ScopedAttribute(ids, m_delimiter));
     vector< pair<string,string> >& dest = scoped->getValues();
     vector<XMLObject*>::const_iterator v,stop;
 
@@ -123,7 +122,7 @@ shibsp::Attribute* ScopedAttributeDecoder::decode(
                         delete[] scope;
                     }
                     else {
-                        scope = strchr(val, m_delimeter);
+                        scope = strchr(val, m_delimiter);
                         if (scope) {
                             *scope++ = 0;
                             if (*scope)
@@ -146,7 +145,7 @@ shibsp::Attribute* ScopedAttributeDecoder::decode(
             }
         }
 
-        return dest.empty() ? NULL : scoped.release();
+        return dest.empty() ? NULL : _decode(scoped.release());
     }
 
     const NameID* saml2name = dynamic_cast<const NameID*>(xmlObject);
@@ -175,8 +174,8 @@ shibsp::Attribute* ScopedAttributeDecoder::decode(
         }
     }
 
-    if (val && *val && *val!=m_delimeter) {
-        scope = strchr(val, m_delimeter);
+    if (val && *val && *val!=m_delimiter) {
+        scope = strchr(val, m_delimiter);
         if (scope) {
             *scope++ = 0;
             if (*scope)
@@ -185,12 +184,12 @@ shibsp::Attribute* ScopedAttributeDecoder::decode(
                 log.warn("ignoring NameID with no scope");
         }
         else {
-            log.warn("ignoring NameID with no scope delimiter (%c)", m_delimeter);
+            log.warn("ignoring NameID with no scope delimiter (%c)", m_delimiter);
         }
     }
     else {
         log.warn("ignoring empty NameID");
     }
     delete[] val;
-    return dest.empty() ? NULL : scoped.release();
+    return dest.empty() ? NULL : _decode(scoped.release());
 }
