@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -57,7 +57,6 @@ shibsp::Attribute* StringAttributeDecoder::decode(
 {
     char* val;
     auto_ptr<SimpleAttribute> simple(new SimpleAttribute(ids));
-    simple->setCaseSensitive(m_caseSensitive);
     vector<string>& dest = simple->getValues();
     vector<XMLObject*>::const_iterator v,stop;
 
@@ -111,7 +110,7 @@ shibsp::Attribute* StringAttributeDecoder::decode(
             }
         }
 
-        return dest.empty() ? NULL : simple.release();
+        return dest.empty() ? NULL : _decode(simple.release());
     }
 
     const NameID* saml2name = dynamic_cast<const NameID*>(xmlObject);
@@ -145,5 +144,5 @@ shibsp::Attribute* StringAttributeDecoder::decode(
     else
         log.warn("ignoring empty NameID");
     delete[] val;
-    return dest.empty() ? NULL : simple.release();
+    return dest.empty() ? NULL : _decode(simple.release());
 }
